@@ -26,7 +26,6 @@
 #include <QItemSelection>
 #include <QIcon>
 #include <QImage>
-#include <libfm/fm.h>
 #include <QList>
 #include <vector>
 #include <utility>
@@ -53,6 +52,7 @@ public:
         ColumnFileType,
         ColumnFileSize,
         ColumnFileMTime,
+        ColumnFileDTime,
         ColumnFileOwner,
         ColumnFileGroup,
         NumOfColumns
@@ -60,7 +60,7 @@ public:
 
 public:
     explicit FolderModel();
-    virtual ~FolderModel();
+    ~FolderModel() override;
 
     const std::shared_ptr<Fm::Folder>& folder() const {
         return folder_;
@@ -72,29 +72,32 @@ public:
         return folder_ ? folder_->path() : Fm::FilePath();
     }
 
-    int rowCount(const QModelIndex& parent = QModelIndex()) const;
-    int columnCount(const QModelIndex& parent) const;
-    QVariant data(const QModelIndex& index, int role) const;
-    QVariant headerData(int section, Qt::Orientation orientation, int role) const;
-    QModelIndex index(int row, int column, const QModelIndex& parent = QModelIndex()) const;
-    QModelIndex parent(const QModelIndex& index) const;
+    int rowCount(const QModelIndex& parent = QModelIndex()) const override;
+    int columnCount(const QModelIndex& parent) const override;
+    QVariant data(const QModelIndex& index, int role) const override;
+    QVariant headerData(int section, Qt::Orientation orientation, int role) const override;
+    QModelIndex index(int row, int column, const QModelIndex& parent = QModelIndex()) const override;
+    QModelIndex parent(const QModelIndex& index) const override;
     // void sort(int column, Qt::SortOrder order = Qt::AscendingOrder);
 
-    Qt::ItemFlags flags(const QModelIndex& index) const;
+    Qt::ItemFlags flags(const QModelIndex& index) const override;
 
-    virtual QStringList mimeTypes() const;
-    virtual QMimeData* mimeData(const QModelIndexList& indexes) const;
-    virtual Qt::DropActions supportedDropActions() const;
-    virtual bool dropMimeData(const QMimeData* data, Qt::DropAction action, int row, int column, const QModelIndex& parent);
+    QStringList mimeTypes() const override;
+    QMimeData* mimeData(const QModelIndexList& indexes) const override;
+    Qt::DropActions supportedDropActions() const override;
+    bool dropMimeData(const QMimeData* data, Qt::DropAction action, int row, int column, const QModelIndex& parent) override;
 
     std::shared_ptr<const Fm::FileInfo> fileInfoFromIndex(const QModelIndex& index) const;
+    std::shared_ptr<const Fm::FileInfo> fileInfoFromPath(const Fm::FilePath& path) const;
     FolderModelItem* itemFromIndex(const QModelIndex& index) const;
     QImage thumbnailFromIndex(const QModelIndex& index, int size);
 
     void cacheThumbnails(int size);
     void releaseThumbnails(int size);
 
-    void setCutFiles(const QItemSelection& selection);
+    void setShowFullName(bool fullName) {
+        showFullNames_ = fullName;
+    }
 
 Q_SIGNALS:
     void thumbnailLoaded(const QModelIndex& index, int size);
@@ -113,14 +116,19 @@ protected Q_SLOTS:
     void onThumbnailJobFinished();
     void loadPendingThumbnails();
 
+    void onClipboardDataChange();
+
 protected:
     void queueLoadThumbnail(const std::shared_ptr<const Fm::FileInfo>& file, int size);
     void insertFiles(int row, const Fm::FileInfoList& files);
     void removeAll();
-    QList<FolderModelItem>::iterator findItemByPath(const Fm::FilePath& path, int* row);
     QList<FolderModelItem>::iterator findItemByName(const char* name, int* row);
     QList<FolderModelItem>::iterator findItemByFileInfo(const Fm::FileInfo* info, int* row);
 
+private:
+    QString makeTooltip(FolderModelItem* item) const;
+    void updateCutFilesSet();
+
 private:
 
     struct ThumbnailData {
@@ -140,6 +148,13 @@ private:
     bool hasPendingThumbnailHandler_;
     std::vector<Fm::ThumbnailJob*> pendingThumbnailJobs_;
     std::forward_list<ThumbnailData> thumbnailData_;
+
+    bool showFullNames_;
+
+    bool isLoaded_;
+
+    bool hasCutfile_;
+    HashSet cutFilesHashSet_;
 };
 
 }
